/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package jalview.appletgui;

import jalview.schemes.*;
import jalview.datamodel.*;
import jalview.analysis.*;
import jalview.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.net.URL;


public class AlignFrame extends Frame
{
  final AlignmentPanel alignPanel;
  public final AlignViewport viewport;
  int NEW_WINDOW_WIDTH = 700;
  int NEW_WINDOW_HEIGHT = 500;
  jalview.bin.JalviewLite applet;


  public AlignFrame(AlignmentI al, jalview.bin.JalviewLite applet, String title)
  {
    try{
      jbInit();
    }catch(Exception ex)
    {
      ex.printStackTrace();
    }

    this.applet = applet;
    viewport = new AlignViewport(al, applet);
    alignPanel = new AlignmentPanel(this, viewport);
    add(alignPanel);
    alignPanel.validate();

    if(applet!=null)
    {
      String param = applet.getParameter("sortBy");
      if (param != null)
      {
        if (param.equalsIgnoreCase("Id"))
          sortIDMenuItem_actionPerformed(null);
        else if (param.equalsIgnoreCase("Pairwise Identity"))
          sortPairwiseMenuItem_actionPerformed(null);
      }

      param = applet.getParameter("wrap");
      if (param != null)
      {
        if (param.equalsIgnoreCase("true"))
        {
          wrapMenuItem.setState(true);
          wrapMenuItem_actionPerformed(null);
        }
      }

      try
      {
        param = applet.getParameter("windowWidth");
        if (param != null)
        {
          int width = Integer.parseInt(param);
          NEW_WINDOW_WIDTH = width;
        }
        param = applet.getParameter("windowHeight");
        if (param != null)
        {
          int height = Integer.parseInt(param);
          NEW_WINDOW_HEIGHT = height;
        }
      }
      catch (Exception ex)
      {}

    }

   MyKeyAdapter keyAdapter = new MyKeyAdapter();
   //Some JVMS send keyevents to Top frame or lowest panel,
   //Havent worked out why yet. So add to both this frame and seqCanvas for now
   this.addKeyListener(keyAdapter);
   alignPanel.seqPanel.seqCanvas.addKeyListener(keyAdapter);
   alignPanel.idPanel.idCanvas.addKeyListener(keyAdapter);

    this.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
          {
            closeMenuItem_actionPerformed(null);
          }
    });

    viewport.addPropertyChangeListener(new java.beans.PropertyChangeListener()
    {
     public void propertyChange(java.beans.PropertyChangeEvent evt)
     {
       if (evt.getPropertyName().equals("alignment"))
       {
         alignmentChanged();
       }
     }
   });

   jalview.bin.JalviewLite.addFrame(this, title,
                                    NEW_WINDOW_WIDTH,
                                    NEW_WINDOW_HEIGHT);
  }

  public AlignViewport getAlignViewport()
  {
    return viewport;
  }

  public SeqCanvas getSeqcanvas()
  {
    return alignPanel.seqPanel.seqCanvas;
  }


  /**
   * DOCUMENT ME!
   *
   * @param String DOCUMENT ME!
   */

  public void parseFeaturesFile(String file)
  {
    try
    {
      URL url = new URL(file);

      BufferedReader in =  new BufferedReader(new InputStreamReader(url.openStream()));
      SequenceI seq = null;
      String line, type, desc, token;

      int index, start, end;
      StringTokenizer st;
      SequenceFeature sf;
      FeatureRenderer fr = alignPanel.seqPanel.seqCanvas.getFeatureRenderer();
      int lineNo = 0;
      while ( (line = in.readLine()) != null)
      {
        lineNo++;
        st = new StringTokenizer(line, "\t");
        if (st.countTokens() == 2)
        {
          type = st.nextToken();
          UserColourScheme ucs = new UserColourScheme(st.nextToken());
          fr.setColour(type, ucs.findColour("A"));
          continue;
        }

        while (st.hasMoreElements())
        {
          desc = st.nextToken();
          token = st.nextToken();
          if (!token.equals("ID_NOT_SPECIFIED"))
          {
            index = viewport.alignment.findIndex(viewport.alignment.findName(
                token));
            st.nextToken();
          }
          else
          {
            index = Integer.parseInt(st.nextToken());
          }

          start = Integer.parseInt(st.nextToken());
          end = Integer.parseInt(st.nextToken());

          seq = viewport.alignment.getSequenceAt(index);
          start = seq.findIndex(start) - 1;
          end = seq.findIndex(end) - 1;

          type = st.nextToken();

          if(fr.getColour(type)==null)
          {
            // Probably the old style groups file
            UserColourScheme ucs = new UserColourScheme(type);
            fr.setColour(type, ucs.findColour("A"));
          }

          sf = new SequenceFeature(type, desc, "", start, end);

          seq.addSequenceFeature(sf);
        }
      }

      viewport.showSequenceFeatures = true;

      alignPanel.repaint();

    }
    catch (Exception ex)
    {
      System.out.println("Error parsing groups file: " + ex);
    }
  }


  class MyKeyAdapter extends KeyAdapter
  {

    public void keyPressed(KeyEvent evt)
    {
      switch (evt.getKeyCode())
      {
        case 27: // escape key
          deselectAllSequenceMenuItem_actionPerformed(null);
          break;
        case KeyEvent.VK_X:
          if (evt.isControlDown() || evt.isMetaDown())
          {
            cut_actionPerformed(null);
          }
          break;
        case KeyEvent.VK_C:
          if (evt.isControlDown() || evt.isMetaDown())
          {
            copy_actionPerformed(null);
          }
          break;
        case KeyEvent.VK_V:
          if (evt.isControlDown() || evt.isMetaDown())
          {
            paste(true);
          }
          break;
        case KeyEvent.VK_A:
          if (evt.isControlDown() || evt.isMetaDown())
          {
            selectAllSequenceMenuItem_actionPerformed(null);
          }
          break;
        case KeyEvent.VK_DOWN:
          moveSelectedSequences(false);
          break;
        case KeyEvent.VK_UP:
          moveSelectedSequences(true);
          break;
        case KeyEvent.VK_F:
          if (evt.isControlDown())
          {
            findMenuItem_actionPerformed(null);
          }
          break;
        case KeyEvent.VK_BACK_SPACE:
        case KeyEvent.VK_DELETE:
          cut_actionPerformed(null);
          break;

      }
    }
  }


  public void inputText_actionPerformed(ActionEvent e)
  {
    CutAndPasteTransfer cap = new CutAndPasteTransfer(true, applet);
    Frame frame = new Frame();
    frame.add(cap);
    jalview.bin.JalviewLite.addFrame(frame, "Cut & Paste Input", 500, 500);
  }

  protected void outputText_actionPerformed(ActionEvent e)
  {
    CutAndPasteTransfer cap = new CutAndPasteTransfer(false, applet);
    Frame frame = new Frame();
    frame.add(cap);
    jalview.bin.JalviewLite.addFrame(frame,
                                     "Alignment output - " + e.getActionCommand(),
                                     600, 500);
    cap.setText(new AppletFormatAdapter().formatSequences(e.getActionCommand(),
                                              viewport.getAlignment().getSequences(),
                                                      viewport.showJVSuffix));
  }

  public void closeMenuItem_actionPerformed(ActionEvent e)
  {
    PaintRefresher.components.remove(viewport.alignment);
    if(PaintRefresher.components.size()==0 && applet==null)
      System.exit(0);
  }

  Stack historyList = new Stack();
  Stack redoList = new Stack();

  void updateEditMenuBar()
  {
    if (historyList.size() > 0)
    {
      undoMenuItem.setEnabled(true);
      HistoryItem hi = (HistoryItem) historyList.peek();
      undoMenuItem.setLabel("Undo " + hi.getDescription());
    }
    else
    {
      undoMenuItem.setEnabled(false);
      undoMenuItem.setLabel("Undo");
    }

    if (redoList.size() > 0)
    {
      redoMenuItem.setEnabled(true);
      HistoryItem hi = (HistoryItem) redoList.peek();
      redoMenuItem.setLabel("Redo " + hi.getDescription());
    }
    else
    {
      redoMenuItem.setEnabled(false);
      redoMenuItem.setLabel("Redo");
    }
  }

  public void addHistoryItem(HistoryItem hi)
  {
    historyList.push(hi);
    updateEditMenuBar();
  }

  protected void undoMenuItem_actionPerformed(ActionEvent e)
  {
    HistoryItem hi = (HistoryItem) historyList.pop();
    redoList.push(new HistoryItem(hi.getDescription(), viewport.alignment,
                                  HistoryItem.HIDE));
    restoreHistoryItem(hi);
    viewport.firePropertyChange("alignment", null, viewport.getAlignment().getSequences());
  }

  protected void redoMenuItem_actionPerformed(ActionEvent e)
  {
    HistoryItem hi = (HistoryItem) redoList.pop();
    restoreHistoryItem(hi);
    updateEditMenuBar();
    viewport.firePropertyChange("alignment", null, viewport.getAlignment().getSequences());
  }

  // used by undo and redo
  void restoreHistoryItem(HistoryItem hi)
  {
    if (hi.getType() == HistoryItem.SORT)
    {
      for (int i = 0; i < hi.getSequences().size(); i++)
      {
        viewport.alignment.getSequences().setElementAt(hi.getSequences().
            elementAt(i), i);
      }
    }
    else
    {
      for (int i = 0; i < hi.getSequences().size(); i++)
      {
        SequenceI restore = (SequenceI) hi.getSequences().elementAt(i);
        if (restore.getLength() == 0)
        {
          // log.System.out.println(hi.getHidden().elementAt(i));
          restore.setSequence(hi.getHidden().elementAt(i).toString());
          viewport.alignment.getSequences().insertElementAt(
              restore,
              hi.getAlignIndex(i));
        }
        else
        {
          restore.setSequence(hi.getHidden().elementAt(i).toString());
        }
      }
      if (hi.getType() == HistoryItem.PASTE)
      {
        for (int i = viewport.alignment.getHeight() - 1;
             i > hi.getSequences().size() - 1; i--)
        {
          viewport.alignment.deleteSequence(i);
        }
      }
    }

    updateEditMenuBar();

    viewport.firePropertyChange("alignment", null, viewport.getAlignment().getSequences());
  }

  public void moveSelectedSequences(boolean up)
  {
    SequenceGroup sg = viewport.getSelectionGroup();
    if (sg == null)
    {
      return;
    }

    if (up)
    {
      for (int i = 1; i < viewport.alignment.getHeight(); i++)
      {
        SequenceI seq = viewport.alignment.getSequenceAt(i);
        if (!sg.sequences.contains(seq))
        {
          continue;
        }

        SequenceI temp = viewport.alignment.getSequenceAt(i - 1);
        if (sg.sequences.contains(temp))
        {
          continue;
        }

        viewport.alignment.getSequences().setElementAt(temp, i);
        viewport.alignment.getSequences().setElementAt(seq, i - 1);
      }
    }
    else
    {
      for (int i = viewport.alignment.getHeight() - 2; i > -1; i--)
      {
        SequenceI seq = viewport.alignment.getSequenceAt(i);
        if (!sg.sequences.contains(seq))
        {
          continue;
        }

        SequenceI temp = viewport.alignment.getSequenceAt(i + 1);
        if (sg.sequences.contains(temp))
        {
          continue;
        }

        viewport.alignment.getSequences().setElementAt(temp, i);
        viewport.alignment.getSequences().setElementAt(seq, i + 1);
      }
    }

    alignPanel.repaint();
  }

  StringBuffer copiedSequences;
  protected void copy_actionPerformed(ActionEvent e)
  {
    if (viewport.getSelectionGroup() == null)
    {
      return;
    }

    SequenceGroup sg = viewport.getSelectionGroup();
    copiedSequences = new StringBuffer();
    Hashtable orderedSeqs = new Hashtable();
    for (int i = 0; i < sg.getSize(); i++)
    {
      SequenceI seq = sg.getSequenceAt(i);
      int index = viewport.alignment.findIndex(seq);
      orderedSeqs.put(index + "", seq);
    }

    int index = 0, startRes, endRes;
    char ch;

    for (int i = 0; i < sg.getSize(); i++)
    {
        SequenceI seq = null;

        while (seq == null)
        {
            if (orderedSeqs.containsKey(index + ""))
            {
                seq = (SequenceI) orderedSeqs.get(index + "");
                index++;

                break;
            }
            else
            {
                index++;
            }
        }

        //FIND START RES
        //Returns residue following index if gap
        startRes = seq.findPosition(sg.getStartRes());

        //FIND END RES
        //Need to find the residue preceeding index if gap
        endRes = 0;

        for (int j = 0; j < sg.getEndRes()+1 && j < seq.getLength(); j++)
        {
          ch = seq.getCharAt(j);
          if (!jalview.util.Comparison.isGap( (ch)))
          {
            endRes++;
          }
        }

        if(endRes>0)
        {
          endRes += seq.getStart() -1;
        }

        copiedSequences.append(seq.getName() + "\t" +
            startRes + "\t" +
            endRes + "\t" +
            seq.getSequence(sg.getStartRes(),
                sg.getEndRes() + 1) + "\n");
    }

  }

  protected void pasteNew_actionPerformed(ActionEvent e)
  {
    paste(true);
  }

  protected void pasteThis_actionPerformed(ActionEvent e)
  {
    addHistoryItem(new HistoryItem("Paste Sequences", viewport.alignment,
                                   HistoryItem.PASTE));
    paste(false);
  }

  void paste(boolean newAlignment)
  {
    try
    {
      if (copiedSequences == null)
      {
        return;
      }

      StringTokenizer st = new StringTokenizer(copiedSequences.toString());
      Vector seqs = new Vector();
      while (st.hasMoreElements())
      {
        String name = st.nextToken();
        int start = Integer.parseInt(st.nextToken());
        int end = Integer.parseInt(st.nextToken());
        Sequence sequence = new Sequence(name, st.nextToken(), start, end);

        if (!newAlignment)
        {
          viewport.alignment.addSequence(sequence);
        }
        else
        {
          seqs.addElement(sequence);
        }
      }

      if (newAlignment)
      {
        SequenceI[] newSeqs = new SequenceI[seqs.size()];
        for (int i = 0; i < seqs.size(); i++)
        {
          newSeqs[i] = (SequenceI) seqs.elementAt(i);
        }

        String newtitle = new String("Copied sequences");
        if (getTitle().startsWith("Copied sequences"))
        {
          newtitle = getTitle();
        }
        else
        {
          newtitle = newtitle.concat("- from " + getTitle());
        }
        AlignFrame af = new AlignFrame(new Alignment(newSeqs), applet, newtitle);

        jalview.bin.JalviewLite.addFrame(af, newtitle, NEW_WINDOW_WIDTH,
                                         NEW_WINDOW_HEIGHT);
      }
      else
      {
        viewport.setEndSeq(viewport.alignment.getHeight());
        viewport.alignment.getWidth();
        viewport.firePropertyChange("alignment", null, viewport.getAlignment().getSequences());
      }

    }
    catch (Exception ex)
    {} // could be anything being pasted in here

  }

  protected void cut_actionPerformed(ActionEvent e)
  {
    copy_actionPerformed(null);
    delete_actionPerformed(null);
  }

  protected void delete_actionPerformed(ActionEvent e)
  {
    addHistoryItem(new HistoryItem("Delete Sequences", viewport.alignment,
                                   HistoryItem.HIDE));
    if (viewport.getSelectionGroup() == null)
    {
      return;
    }


    SequenceGroup sg = viewport.getSelectionGroup();
    boolean allSequences = false;
    if(sg.sequences.size()==viewport.alignment.getHeight())
          allSequences = true;

    for (int i = 0; i < sg.sequences.size(); i++)
    {
      SequenceI seq = sg.getSequenceAt(i);
      int index = viewport.getAlignment().findIndex(seq);
      seq.deleteChars(sg.getStartRes(), sg.getEndRes() + 1);

      // If the cut affects all sequences, remove highlighted columns
      if (allSequences)
      {
        viewport.getColumnSelection().removeElements(sg.getStartRes(),
                                                     sg.getEndRes() + 1);
      }


      if (seq.getSequence().length() < 1)
      {
        viewport.getAlignment().deleteSequence(seq);
      }
      else
      {
        viewport.getAlignment().getSequences().setElementAt(seq, index);
      }
    }

    viewport.setSelectionGroup(null);
    viewport.alignment.deleteGroup(sg);
    viewport.resetSeqLimits(alignPanel.seqPanel.seqCanvas.getSize().height);
    if (viewport.getAlignment().getHeight() < 1)
    {
      try
      {
        this.setVisible(false);
      }
      catch (Exception ex)
      {}
    }
    viewport.firePropertyChange("alignment", null, viewport.getAlignment().getSequences());

  }

  protected void deleteGroups_actionPerformed(ActionEvent e)
  {
    viewport.alignment.deleteAllGroups();
    viewport.setSelectionGroup(null);

    alignPanel.repaint();
  }

  public void selectAllSequenceMenuItem_actionPerformed(ActionEvent e)
  {
    SequenceGroup sg = new SequenceGroup();
    for (int i = 0; i < viewport.getAlignment().getSequences().size(); i++)
    {
      sg.addSequence(viewport.getAlignment().getSequenceAt(i), false);
    }
    sg.setEndRes(viewport.alignment.getWidth()-1);
    viewport.setSelectionGroup(sg);
    alignPanel.repaint();
    PaintRefresher.Refresh(null, viewport.alignment);
  }

  public void deselectAllSequenceMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setSelectionGroup(null);
    viewport.getColumnSelection().clear();
    viewport.setSelectionGroup(null);
    alignPanel.seqPanel.seqCanvas.highlightSearchResults(null);
    alignPanel.repaint();
    PaintRefresher.Refresh(null, viewport.alignment);
  }

  public void invertSequenceMenuItem_actionPerformed(ActionEvent e)
  {
    SequenceGroup sg = viewport.getSelectionGroup();
    for (int i = 0; i < viewport.getAlignment().getSequences().size(); i++)
    {
      sg.addOrRemove(viewport.getAlignment().getSequenceAt(i), false);
    }

    PaintRefresher.Refresh(null, viewport.alignment);
  }

  public void remove2LeftMenuItem_actionPerformed(ActionEvent e)
  {
    ColumnSelection colSel = viewport.getColumnSelection();
    if (colSel.size() > 0)
    {
      addHistoryItem(new HistoryItem("Remove Left", viewport.alignment,
                                     HistoryItem.HIDE));
      int min = colSel.getMin();
      viewport.getAlignment().trimLeft(min);
      colSel.compensateForEdit(0, min);

      if (viewport.getSelectionGroup() != null)
      {
        viewport.getSelectionGroup().adjustForRemoveLeft(min);
      }

      Vector groups = viewport.alignment.getGroups();
      for (int i = 0; i < groups.size(); i++)
      {
        SequenceGroup sg = (SequenceGroup) groups.elementAt(i);
        if (!sg.adjustForRemoveLeft(min))
        {
          viewport.alignment.deleteGroup(sg);
        }
      }
      viewport.firePropertyChange("alignment", null, viewport.getAlignment().getSequences());
    }
  }

  public void remove2RightMenuItem_actionPerformed(ActionEvent e)
  {
    ColumnSelection colSel = viewport.getColumnSelection();
    if (colSel.size() > 0)
    {
      addHistoryItem(new HistoryItem("Remove Right", viewport.alignment,
                                     HistoryItem.HIDE));
      int max = colSel.getMax();
      viewport.getAlignment().trimRight(max);
      if (viewport.getSelectionGroup() != null)
      {
        viewport.getSelectionGroup().adjustForRemoveRight(max);
      }

      Vector groups = viewport.alignment.getGroups();
      for (int i = 0; i < groups.size(); i++)
      {
        SequenceGroup sg = (SequenceGroup) groups.elementAt(i);
        if (!sg.adjustForRemoveRight(max))
        {
          viewport.alignment.deleteGroup(sg);
        }
      }
      viewport.firePropertyChange("alignment", null, viewport.getAlignment().getSequences());
    }

  }

  public void removeGappedColumnMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem(new HistoryItem("Remove Gapped Columns",
                                   viewport.alignment,
                                   HistoryItem.HIDE));

    //This is to maintain viewport position on first residue
    //of first sequence
    SequenceI seq = viewport.alignment.getSequenceAt(0);
    int startRes = seq.findPosition(viewport.startRes);

    viewport.getAlignment().removeGaps();

    viewport.setStartRes(seq.findIndex(startRes)-1);

    viewport.firePropertyChange("alignment", null, viewport.getAlignment().getSequences());
  }

  public void removeAllGapsMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem(new HistoryItem("Remove Gaps", viewport.alignment,
                                   HistoryItem.HIDE));

    //This is to maintain viewport position on first residue
    //of first sequence
    SequenceI seq = viewport.alignment.getSequenceAt(0);
    int startRes = seq.findPosition(viewport.startRes);

    SequenceI current;
    int jSize;

    Vector seqs=null;

    int start=0, end = viewport.alignment.getWidth();

    if (viewport.getSelectionGroup() != null
        && viewport.getSelectionGroup().sequences != null
        && viewport.getSelectionGroup().sequences.size()>0)
    {
      seqs = viewport.getSelectionGroup().sequences;
      start = viewport.getSelectionGroup().getStartRes();
      end = viewport.getSelectionGroup().getEndRes()+1;
    }
    else
      seqs = viewport.alignment.getSequences();

    for (int i = 0; i < seqs.size(); i++)
    {
      current = (SequenceI) seqs.elementAt(i);
      jSize = current.getLength();

      // Removing a range is much quicker than removing gaps
      // one by one for long sequences
      int j = start;
      int rangeStart=-1, rangeEnd=-1;

      do
      {
        if (jalview.util.Comparison.isGap(current.getCharAt(j)))
        {
          if(rangeStart==-1)
           {
             rangeStart = j;
             rangeEnd = j+1;
           }
           else
           {
             rangeEnd++;
           }
           j++;
        }
        else
        {
          if(rangeStart>-1)
          {
            current.deleteChars(rangeStart, rangeEnd);
            j-=rangeEnd-rangeStart;
            jSize-=rangeEnd-rangeStart;
            rangeStart = -1;
            rangeEnd = -1;
          }
          else
            j++;
        }
      }
      while (j < end && j < jSize);
      if(rangeStart>-1)
      {
       current.deleteChars(rangeStart, rangeEnd);
      }
    }
    viewport.setStartRes(seq.findIndex(startRes)-1);
    viewport.firePropertyChange("alignment", null, viewport.getAlignment().getSequences());
  }

  public void alignmentChanged()
  {
    viewport.updateConsensus();
    viewport.updateConservation ();
    resetAllColourSchemes();
    if(alignPanel.overviewPanel!=null)
      alignPanel.overviewPanel.updateOverviewImage();
    alignPanel.repaint();
  }

  void resetAllColourSchemes()
  {
    ColourSchemeI cs = viewport.globalColourScheme;
    if(cs!=null)
    {
      if (cs instanceof ClustalxColourScheme)
      {
        ( (ClustalxColourScheme) viewport.getGlobalColourScheme()).
            resetClustalX(viewport.alignment.getSequences(),
                          viewport.alignment.getWidth());
      }

      cs.setConsensus(viewport.vconsensus);
      if (cs.conservationApplied())
      {
        Alignment al = (Alignment) viewport.alignment;
        Conservation c = new Conservation("All",
                                          ResidueProperties.propHash, 3,
                                          al.getSequences(), 0,
                                          al.getWidth() - 1);
        c.calculate();
        c.verdict(false, viewport.ConsPercGaps);

        cs.setConservation(c);
      }
    }

    int s, sSize = viewport.alignment.getGroups().size();
    for(s=0; s<sSize; s++)
    {
      SequenceGroup sg = (SequenceGroup)viewport.alignment.getGroups().elementAt(s);
      if(sg.cs!=null && sg.cs instanceof ClustalxColourScheme)
      {
        ((ClustalxColourScheme)sg.cs).resetClustalX(sg.sequences, sg.getWidth());
      }
      sg.recalcConservation();
    }
  }



  public void findMenuItem_actionPerformed(ActionEvent e)
  {
    new Finder(alignPanel);
  }

  public void font_actionPerformed(ActionEvent e)
  {
    new FontChooser(alignPanel);
  }


  public void seqLimits_itemStateChanged(ItemEvent e)
  {
    viewport.setShowJVSuffix(seqLimits.getState());
    alignPanel.fontChanged();
    alignPanel.repaint();
  }


  protected void colourTextMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setColourText(colourTextMenuItem.getState());
    alignPanel.repaint();
  }

  protected void wrapMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setWrapAlignment(wrapMenuItem.getState());
    alignPanel.setWrapAlignment(wrapMenuItem.getState());
    scaleAbove.setEnabled(wrapMenuItem.getState());
    scaleLeft.setEnabled(wrapMenuItem.getState());
    scaleRight.setEnabled(wrapMenuItem.getState());
    alignPanel.repaint();
  }

  protected void scaleAbove_actionPerformed(ActionEvent e)
  {
    viewport.setScaleAboveWrapped(scaleAbove.getState());
    alignPanel.repaint();
  }

  protected void scaleLeft_actionPerformed(ActionEvent e)
  {
    viewport.setScaleLeftWrapped(scaleLeft.getState());
    alignPanel.repaint();
  }

  protected void scaleRight_actionPerformed(ActionEvent e)
  {
    viewport.setScaleRightWrapped(scaleRight.getState());
    alignPanel.repaint();
  }

  public void viewBoxesMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setShowBoxes(viewBoxesMenuItem.getState());
    alignPanel.repaint();
  }

  public void viewTextMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setShowText(viewTextMenuItem.getState());
    alignPanel.repaint();
  }

  protected void renderGapsMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setRenderGaps(renderGapsMenuItem.getState());
    alignPanel.repaint();
  }

  public void annotationPanelMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setShowAnnotation(annotationPanelMenuItem.getState());
    alignPanel.setAnnotationVisible(annotationPanelMenuItem.getState());
  }

  public void overviewMenuItem_actionPerformed(ActionEvent e)
  {
    if (alignPanel.overviewPanel != null)
    {
      return;
    }

    Frame frame = new Frame();
    OverviewPanel overview = new OverviewPanel(alignPanel);
    frame.add(overview);
    // +50 must allow for applet frame window
    jalview.bin.JalviewLite.addFrame(frame, "Overview " + this.getTitle(),
                                     overview.preferredSize().width,
                                     overview.preferredSize().height + 50);

    frame.pack();
    frame.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        alignPanel.setOverviewPanel(null);
      };
    });

    alignPanel.setOverviewPanel(overview);

  }

  protected void noColourmenuItem_actionPerformed(ActionEvent e)
  {
    changeColour(null);
  }

  public void clustalColour_actionPerformed(ActionEvent e)
  {
    abovePIDThreshold.setState(false);
    changeColour(new ClustalxColourScheme(viewport.alignment.getSequences(),
                                          viewport.alignment.getWidth()));
  }

  public void zappoColour_actionPerformed(ActionEvent e)
  {
    changeColour(new ZappoColourScheme());
  }

  public void taylorColour_actionPerformed(ActionEvent e)
  {
    changeColour(new TaylorColourScheme());
  }

  public void hydrophobicityColour_actionPerformed(ActionEvent e)
  {
    changeColour(new HydrophobicColourScheme());
  }

  public void helixColour_actionPerformed(ActionEvent e)
  {
    changeColour(new HelixColourScheme());
  }

  public void strandColour_actionPerformed(ActionEvent e)
  {
    changeColour(new StrandColourScheme());
  }

  public void turnColour_actionPerformed(ActionEvent e)
  {
    changeColour(new TurnColourScheme());
  }

  public void buriedColour_actionPerformed(ActionEvent e)
  {
    changeColour(new BuriedColourScheme());
  }

  public void nucleotideColour_actionPerformed(ActionEvent e)
  {
    changeColour(new NucleotideColourScheme());
  }

  protected void applyToAllGroups_actionPerformed(ActionEvent e)
  {
    viewport.setColourAppliesToAllGroups(applyToAllGroups.getState());
  }

  void changeColour(ColourSchemeI cs)
  {
    int threshold = 0;

    if(cs!=null)
    {
      if (viewport.getAbovePIDThreshold())
      {
        threshold = SliderPanel.setPIDSliderSource(alignPanel, cs, "Background");

        cs.setThreshold(threshold, viewport.getIgnoreGapsConsensus());

        viewport.setGlobalColourScheme(cs);
      }
      else
      {
        cs.setThreshold(0, viewport.getIgnoreGapsConsensus());
      }

      if (viewport.getConservationSelected())
      {

        Alignment al = (Alignment) viewport.alignment;
        Conservation c = new Conservation("All",
                                          ResidueProperties.propHash, 3,
                                          al.getSequences(), 0,
                                          al.getWidth() - 1);

        c.calculate();
        c.verdict(false, viewport.ConsPercGaps);

        cs.setConservation(c);

        cs.setConservationInc(SliderPanel.setConservationSlider(alignPanel, cs,
            "Background"));

      }
      else
      {
        cs.setConservation(null);
      }

      cs.setConsensus(viewport.vconsensus);

    }
    viewport.setGlobalColourScheme(cs);

    if (viewport.getColourAppliesToAllGroups())
    {
      Vector groups = viewport.alignment.getGroups();
      for (int i = 0; i < groups.size(); i++)
      {
        SequenceGroup sg = (SequenceGroup) groups.elementAt(i);

        if(cs==null)
        {
          sg.cs = null;
          continue;
        }
        if (cs instanceof ClustalxColourScheme)
        {
          sg.cs = new ClustalxColourScheme(sg.sequences, sg.getWidth());
        }
        else
        {
          try
          {
            sg.cs = (ColourSchemeI) cs.getClass().newInstance();
          }
          catch (Exception ex)
          {
            ex.printStackTrace();
            sg.cs = cs;
          }
        }

        if (viewport.getAbovePIDThreshold()
            || cs instanceof PIDColourScheme
            || cs instanceof Blosum62ColourScheme)
        {
          sg.cs.setThreshold(threshold, viewport.getIgnoreGapsConsensus());
          sg.cs.setConsensus(AAFrequency.calculate(sg.sequences, 0, sg.getWidth()));
        }
        else
          sg.cs.setThreshold(0, viewport.getIgnoreGapsConsensus());

        if (viewport.getConservationSelected())
        {
          Conservation c = new Conservation("Group",
                                            ResidueProperties.propHash, 3,
                                            sg.sequences, 0,
                                            viewport.alignment.getWidth() - 1);
          c.calculate();
          c.verdict(false, viewport.ConsPercGaps);
          sg.cs.setConservation(c);
        }
        else
        {
          sg.cs.setConservation(null);
          sg.cs.setThreshold(0, viewport.getIgnoreGapsConsensus());
        }

      }
    }


    if (alignPanel.getOverviewPanel() != null)
    {
      alignPanel.getOverviewPanel().updateOverviewImage();
    }

    alignPanel.repaint();
  }



  protected void modifyPID_actionPerformed(ActionEvent e)
  {
    if (viewport.getAbovePIDThreshold() && viewport.globalColourScheme!=null)
    {
      SliderPanel.setPIDSliderSource(alignPanel, viewport.getGlobalColourScheme(),
                                     "Background");
      SliderPanel.showPIDSlider();
    }
  }

  protected void modifyConservation_actionPerformed(ActionEvent e)
  {
    if (viewport.getConservationSelected() && viewport.globalColourScheme!=null)
    {
      SliderPanel.setConservationSlider(alignPanel, viewport.globalColourScheme,
                                        "Background");
      SliderPanel.showConservationSlider();
    }
  }

  protected void conservationMenuItem_actionPerformed(ActionEvent e)
  {
    viewport.setConservationSelected(conservationMenuItem.getState());

    viewport.setAbovePIDThreshold(false);
    abovePIDThreshold.setState(false);

    changeColour(viewport.getGlobalColourScheme());

    modifyConservation_actionPerformed(null);
  }

  public void abovePIDThreshold_actionPerformed(ActionEvent e)
  {
    viewport.setAbovePIDThreshold(abovePIDThreshold.getState());

    conservationMenuItem.setState(false);
    viewport.setConservationSelected(false);

    changeColour(viewport.getGlobalColourScheme());

    modifyPID_actionPerformed(null);
  }

  public void userDefinedColour_actionPerformed(ActionEvent e)
  {
    new UserDefinedColours(alignPanel, null);
  }

  public void PIDColour_actionPerformed(ActionEvent e)
  {
    changeColour(new PIDColourScheme());
  }

  public void BLOSUM62Colour_actionPerformed(ActionEvent e)
  {
    changeColour(new Blosum62ColourScheme());
  }

  public void sortPairwiseMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem(new HistoryItem("Pairwise Sort", viewport.alignment,
                                   HistoryItem.SORT));
    AlignmentSorter.sortByPID(viewport.getAlignment(),
                              viewport.getAlignment().getSequenceAt(0));
    alignPanel.repaint();
  }

  public void sortIDMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem(new HistoryItem("ID Sort", viewport.alignment,
                                   HistoryItem.SORT));
    AlignmentSorter.sortByID(viewport.getAlignment());
    alignPanel.repaint();
  }

  public void sortGroupMenuItem_actionPerformed(ActionEvent e)
  {
    addHistoryItem(new HistoryItem("Group Sort", viewport.alignment,
                                   HistoryItem.SORT));
    AlignmentSorter.sortByGroup(viewport.getAlignment());
    alignPanel.repaint();

  }

  public void removeRedundancyMenuItem_actionPerformed(ActionEvent e)
  {
    RedundancyPanel sp = new RedundancyPanel(alignPanel);
    Frame frame = new Frame();
    frame.add(sp);
    jalview.bin.JalviewLite.addFrame(frame, "Redundancy threshold selection",
                                     400, 120);

  }

  public void pairwiseAlignmentMenuItem_actionPerformed(ActionEvent e)
  {
    if (viewport.getSelectionGroup().getSize() > 1)
    {
      Frame frame = new Frame();
      frame.add(new PairwiseAlignPanel(alignPanel));
      jalview.bin.JalviewLite.addFrame(frame, "Pairwise Alignment", 600, 500);
    }
  }

  public void PCAMenuItem_actionPerformed(ActionEvent e)
  {
    //are the sequences aligned?
    if (!viewport.alignment.isAligned())
    {
      SequenceI current;
      int Width = viewport.getAlignment().getWidth();

      for (int i = 0; i < viewport.getAlignment().getSequences().size();
           i++)
      {
        current = viewport.getAlignment().getSequenceAt(i);

        if (current.getLength() < Width)
        {
          current.insertCharAt(Width - 1, viewport.getGapCharacter());
        }
      }
      alignPanel.repaint();
    }

    if ( (viewport.getSelectionGroup() != null &&
          viewport.getSelectionGroup().getSize() < 4 &&
          viewport.getSelectionGroup().getSize() > 0)
        || viewport.getAlignment().getHeight() < 4)
    {
      return;
    }

    try
    {
      new PCAPanel(viewport);
    }
    catch (java.lang.OutOfMemoryError ex)
    {
    }

  }

  public void averageDistanceTreeMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("AV", "PID", "Average distance tree using PID");
  }

  public void neighbourTreeMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("NJ", "PID", "Neighbour joining tree using PID");
  }

  protected void njTreeBlosumMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("NJ", "BL", "Neighbour joining tree using BLOSUM62");
  }

  protected void avTreeBlosumMenuItem_actionPerformed(ActionEvent e)
  {
    NewTreePanel("AV", "BL", "Average distance tree using BLOSUM62PID");
  }

  void NewTreePanel(String type, String pwType, String title)
  {
    //are the sequences aligned?
    if (!viewport.alignment.isAligned())
    {
      SequenceI current;
      int Width = viewport.getAlignment().getWidth();

      for (int i = 0; i < viewport.getAlignment().getSequences().size();
           i++)
      {
        current = viewport.getAlignment().getSequenceAt(i);

        if (current.getLength() < Width)
        {
          current.insertCharAt(Width - 1, viewport.getGapCharacter());
        }
      }
      alignPanel.repaint();

    }

    final TreePanel tp;
    if (viewport.getSelectionGroup() != null &&
        viewport.getSelectionGroup().getSize() > 3)
    {
      tp = new TreePanel(viewport, viewport.getSelectionGroup().sequences, type,
                         pwType,
                         0, viewport.alignment.getWidth());
    }
    else
    {
      tp = new TreePanel(viewport, viewport.getAlignment().getSequences(),
                         type, pwType, 0, viewport.alignment.getWidth());
    }

    addTreeMenuItem(tp, title);

    jalview.bin.JalviewLite.addFrame(tp, title, 600, 500);
  }

  public void addTreeMenuItem(final TreePanel treePanel, String title)
  {
    final MenuItem item = new MenuItem(title);
    sortByTreeMenu.add(item);
    item.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        addHistoryItem(new HistoryItem("Sort", viewport.alignment,
                                       HistoryItem.SORT));
        AlignmentSorter.sortByTree(viewport.getAlignment(), treePanel.getTree());
        alignPanel.repaint();
      }
    });

    treePanel.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        sortByTreeMenu.remove(item);
      };
    });
  }

  protected void documentation_actionPerformed(ActionEvent e)
  {
    showURL("http://www.jalview.org/help.html");
  }

  protected void about_actionPerformed(ActionEvent e)
  {

    class AboutPanel extends Canvas
    {
      String version;
      public AboutPanel(String version)
      { this.version = version; }

      public void paint(Graphics g)
      {
        g.setColor(Color.white);
        g.fillRect(0, 0, getSize().width, getSize().height);
        g.setFont(new Font("Helvetica", Font.PLAIN, 12));
        FontMetrics fm = g.getFontMetrics();
        int fh = fm.getHeight();
        int y = 5, x = 7;
        g.setColor(Color.black);
        g.setFont(new Font("Helvetica", Font.BOLD, 14));
        g.drawString("Jalview - Release "+version, 200, y += fh);
        g.setFont(new Font("Helvetica", Font.PLAIN, 12));
        g.drawString("Authors:  Michele Clamp, James Cuff, Steve Searle, Andrew Waterhouse, Jim Procter & Geoff Barton.",
                     x, y += fh * 2);
        g.drawString("Current development managed by Andrew Waterhouse; Barton Group, University of Dundee.",
                     x, y += fh);
        g.drawString(
            "For any issues relating to Jalview, email help@jalview.org", x,
            y += fh);
        g.drawString("If  you use JalView, please cite:", x, y += fh + 8);
        g.drawString("\"Clamp, M., Cuff, J., Searle, S. M. and Barton, G. J. (2004), The Jalview Java Alignment Editor\"",
                     x, y += fh);
        g.drawString("Bioinformatics,  2004 12;426-7.", x, y += fh);
      }
    }

    String version = "test";
    java.net.URL url = getClass().getResource("/.build_properties");
    if (url != null)
    {
      try
      {
        BufferedReader reader = new BufferedReader(new InputStreamReader(
            url.openStream()));
        String line;
        while ( (line = reader.readLine()) != null)
        {
          if (line.indexOf("VERSION") > -1)
          {
            version = line.substring(line.indexOf("=") + 1);
          }
        }
      }
      catch (Exception ex)
      {
        ex.printStackTrace();
      }
    }


    Frame frame = new Frame();
    frame.add(new AboutPanel(version));
    jalview.bin.JalviewLite.addFrame(frame, "Jalview", 580, 200);

  }

  /**
   * Displays the given URL in a new browser window
   *
   * @param url URL to display in browser window.
   * <br>New window will be named "HELP_WINDOW"
   */
  public void showURL(String url)
  {
    showURL(url, "HELP");
  }

  public void showURL(String url, String target)
  {
    if (applet == null)
    {
      System.out.println("Not running as applet - no browser available.");
    }
    else
    {
      try
      {
        applet.getAppletContext().showDocument(new java.net.URL(url),
                                               target);
      }
      catch (Exception ex)
      {
        ex.printStackTrace();
      }
    }
  }


  //////////////////////////////////////////////////////////////////////////////////
  //JBuilder Graphics here

    protected MenuBar alignFrameMenuBar = new MenuBar();
    protected Menu fileMenu = new Menu();
    protected MenuItem closeMenuItem = new MenuItem();
    protected Menu editMenu = new Menu();
    protected Menu viewMenu = new Menu();
    protected Menu colourMenu = new Menu();
    protected Menu calculateMenu = new Menu();
    protected MenuItem selectAllSequenceMenuItem = new MenuItem();
    protected MenuItem deselectAllSequenceMenuItem = new MenuItem();
    protected MenuItem invertSequenceMenuItem = new MenuItem();
    protected MenuItem remove2LeftMenuItem = new MenuItem();
    protected MenuItem remove2RightMenuItem = new MenuItem();
    protected MenuItem removeGappedColumnMenuItem = new MenuItem();
    protected MenuItem removeAllGapsMenuItem = new MenuItem();
    protected CheckboxMenuItem viewBoxesMenuItem = new CheckboxMenuItem();
    protected CheckboxMenuItem viewTextMenuItem = new CheckboxMenuItem();
    protected MenuItem sortPairwiseMenuItem = new MenuItem();
    protected MenuItem sortIDMenuItem = new MenuItem();
    protected MenuItem sortGroupMenuItem = new MenuItem();
    protected MenuItem removeRedundancyMenuItem = new MenuItem();
    protected MenuItem pairwiseAlignmentMenuItem = new MenuItem();
    protected MenuItem PCAMenuItem = new MenuItem();
    protected MenuItem averageDistanceTreeMenuItem = new MenuItem();
    protected MenuItem neighbourTreeMenuItem = new MenuItem();
    BorderLayout borderLayout1 = new BorderLayout();
    public Label statusBar = new Label();
    protected Menu outputTextboxMenu = new Menu();
    protected MenuItem clustalColour = new MenuItem();
    protected MenuItem zappoColour = new MenuItem();
    protected MenuItem taylorColour = new MenuItem();
    protected MenuItem hydrophobicityColour = new MenuItem();
    protected MenuItem helixColour = new MenuItem();
    protected MenuItem strandColour = new MenuItem();
    protected MenuItem turnColour = new MenuItem();
    protected MenuItem buriedColour = new MenuItem();
    protected MenuItem userDefinedColour = new MenuItem();
    protected MenuItem PIDColour = new MenuItem();
    protected MenuItem BLOSUM62Colour = new MenuItem();
    MenuItem njTreeBlosumMenuItem = new MenuItem();
    MenuItem avDistanceTreeBlosumMenuItem = new MenuItem();
    protected CheckboxMenuItem annotationPanelMenuItem = new CheckboxMenuItem();
    protected CheckboxMenuItem colourTextMenuItem = new CheckboxMenuItem();
    MenuItem overviewMenuItem = new MenuItem();
    protected MenuItem undoMenuItem = new MenuItem();
    protected MenuItem redoMenuItem = new MenuItem();
    protected CheckboxMenuItem conservationMenuItem = new CheckboxMenuItem();
    MenuItem noColourmenuItem = new MenuItem();
    protected CheckboxMenuItem wrapMenuItem = new CheckboxMenuItem();
    protected CheckboxMenuItem renderGapsMenuItem = new CheckboxMenuItem();
    MenuItem findMenuItem = new MenuItem();
    Menu searchMenu = new Menu();
    protected CheckboxMenuItem abovePIDThreshold = new CheckboxMenuItem();
    protected MenuItem nucleotideColour = new MenuItem();
    MenuItem deleteGroups = new MenuItem();
    MenuItem delete = new MenuItem();
    MenuItem copy = new MenuItem();
    MenuItem cut = new MenuItem();
    Menu Menu1 = new Menu();
    MenuItem pasteNew = new MenuItem();
    MenuItem pasteThis = new MenuItem();
    protected CheckboxMenuItem applyToAllGroups = new CheckboxMenuItem();
    protected MenuItem font = new MenuItem();
    protected CheckboxMenuItem scaleAbove = new CheckboxMenuItem();
    protected CheckboxMenuItem scaleLeft = new CheckboxMenuItem();
    protected CheckboxMenuItem scaleRight = new CheckboxMenuItem();
    MenuItem modifyPID = new MenuItem();
    MenuItem modifyConservation = new MenuItem();
    protected Menu sortByTreeMenu = new Menu();
    Menu sort = new Menu();
    Menu calculate = new Menu();
    MenuItem inputText = new MenuItem();
    Menu helpMenu = new Menu();
    MenuItem documentation = new MenuItem();
    MenuItem about = new MenuItem();
    protected CheckboxMenuItem seqLimits = new CheckboxMenuItem();

    private void jbInit() throws Exception {

      setMenuBar(alignFrameMenuBar);

      MenuItem item;

      // dynamically fill save as menu with available formats
      for (int i = 0; i < jalview.io.AppletFormatAdapter.formats.size(); i++)
      {

        item = new MenuItem( (String) jalview.io.AppletFormatAdapter.formats.
                            elementAt(
                                i));
        item.addActionListener(new java.awt.event.ActionListener()
        {
          public void actionPerformed(ActionEvent e)
          {
            outputText_actionPerformed(e);
          }
        });

        outputTextboxMenu.add(item);
      }


        fileMenu.setLabel("File");
        closeMenuItem.setLabel("Close");
        closeMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    closeMenuItem_actionPerformed(e);
                }
            });
        editMenu.setLabel("Edit");
        viewMenu.setLabel("View");
        colourMenu.setLabel("Colour");
        calculateMenu.setLabel("Calculate");
        selectAllSequenceMenuItem.setLabel("Select all");
        selectAllSequenceMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    selectAllSequenceMenuItem_actionPerformed(e);
                }
            });
        deselectAllSequenceMenuItem.setLabel("Deselect All");
        deselectAllSequenceMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    deselectAllSequenceMenuItem_actionPerformed(e);
                }
            });
        invertSequenceMenuItem.setLabel("Invert Selection");
        invertSequenceMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    invertSequenceMenuItem_actionPerformed(e);
                }
            });
        remove2LeftMenuItem.setLabel("Remove Left");
        remove2LeftMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    remove2LeftMenuItem_actionPerformed(e);
                }
            });
        remove2RightMenuItem.setLabel("Remove Right");
        remove2RightMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    remove2RightMenuItem_actionPerformed(e);
                }
            });
        removeGappedColumnMenuItem.setLabel("Remove Empty Columns");
        removeGappedColumnMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    removeGappedColumnMenuItem_actionPerformed(e);
                }
            });
        removeAllGapsMenuItem.setLabel("Remove All Gaps");
        removeAllGapsMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    removeAllGapsMenuItem_actionPerformed(e);
                }
            });
        viewBoxesMenuItem.setLabel("Boxes");
        viewBoxesMenuItem.setState(true);
        viewBoxesMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    viewBoxesMenuItem_actionPerformed(null);
                }
            });
        viewTextMenuItem.setLabel("Text");
        viewTextMenuItem.setState(true);
        viewTextMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    viewTextMenuItem_actionPerformed(null);
                }
            });
        sortPairwiseMenuItem.setLabel("by Pairwise Identity");
        sortPairwiseMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    sortPairwiseMenuItem_actionPerformed(e);
                }
            });
        sortIDMenuItem.setLabel("by ID");
        sortIDMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    sortIDMenuItem_actionPerformed(e);
                }
            });
        sortGroupMenuItem.setLabel("by Group");
        sortGroupMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    sortGroupMenuItem_actionPerformed(e);
                }
            });
        removeRedundancyMenuItem.setLabel("Remove Redundancy...");
        removeRedundancyMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    removeRedundancyMenuItem_actionPerformed(e);
                }
            });
        pairwiseAlignmentMenuItem.setLabel("Pairwise Alignments...");
        pairwiseAlignmentMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    pairwiseAlignmentMenuItem_actionPerformed(e);
                }
            });
        PCAMenuItem.setLabel("Principal Component Analysis");
        PCAMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    PCAMenuItem_actionPerformed(e);
                }
            });
        averageDistanceTreeMenuItem.setLabel(
            "Average Distance Using % Identity");
        averageDistanceTreeMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    averageDistanceTreeMenuItem_actionPerformed(e);
                }
            });
        neighbourTreeMenuItem.setLabel("Neighbour Joining Using % Identity");
        neighbourTreeMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    neighbourTreeMenuItem_actionPerformed(e);
                }
            });
        alignFrameMenuBar.setFont(new java.awt.Font("Verdana", 0, 11));
        statusBar.setBackground(Color.white);
        statusBar.setFont(new java.awt.Font("Verdana", 0, 11));
        statusBar.setText("Status bar");
        outputTextboxMenu.setLabel("Output to Textbox");
        clustalColour.setLabel("Clustalx");

        clustalColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    clustalColour_actionPerformed(e);
                }
            });
        zappoColour.setLabel("Zappo");
        zappoColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    zappoColour_actionPerformed(e);
                }
            });
        taylorColour.setLabel("Taylor");
        taylorColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    taylorColour_actionPerformed(e);
                }
            });
        hydrophobicityColour.setLabel("Hydrophobicity");
        hydrophobicityColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    hydrophobicityColour_actionPerformed(e);
                }
            });
        helixColour.setLabel("Helix Propensity");
        helixColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    helixColour_actionPerformed(e);
                }
            });
        strandColour.setLabel("Strand Propensity");
        strandColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    strandColour_actionPerformed(e);
                }
            });
        turnColour.setLabel("Turn Propensity");
        turnColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    turnColour_actionPerformed(e);
                }
            });
        buriedColour.setLabel("Buried Index");
        buriedColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    buriedColour_actionPerformed(e);
                }
            });
        userDefinedColour.setLabel("User Defined...");
        userDefinedColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    userDefinedColour_actionPerformed(e);
                }
            });
        PIDColour.setLabel("Percentage Identity");
        PIDColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    PIDColour_actionPerformed(e);
                }
            });
        BLOSUM62Colour.setLabel("BLOSUM62 Score");
        BLOSUM62Colour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    BLOSUM62Colour_actionPerformed(e);
                }
            });
        avDistanceTreeBlosumMenuItem.setLabel(
            "Average Distance Using BLOSUM62");
        avDistanceTreeBlosumMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    avTreeBlosumMenuItem_actionPerformed(e);
                }
            });
        njTreeBlosumMenuItem.setLabel("Neighbour Joining Using BLOSUM62");
        njTreeBlosumMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    njTreeBlosumMenuItem_actionPerformed(e);
                }
            });
        annotationPanelMenuItem.setActionCommand("");
        annotationPanelMenuItem.setState(true);
        annotationPanelMenuItem.setLabel("Show Annotations");
        annotationPanelMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    annotationPanelMenuItem_actionPerformed(null);
                }
            });
        colourTextMenuItem.setLabel("Colour Text");
        colourTextMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    colourTextMenuItem_actionPerformed(null);
                }
            });
        overviewMenuItem.setLabel("Overview Window");
        overviewMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    overviewMenuItem_actionPerformed(e);
                }
            });
        undoMenuItem.setEnabled(false);
        undoMenuItem.setLabel("Undo");
        undoMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    undoMenuItem_actionPerformed(e);
                }
            });
        redoMenuItem.setEnabled(false);
        redoMenuItem.setLabel("Redo");
        redoMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    redoMenuItem_actionPerformed(e);
                }
            });
        conservationMenuItem.setLabel("by Conservation");
        conservationMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    conservationMenuItem_actionPerformed(null);
                }
            });
        noColourmenuItem.setLabel("None");
        noColourmenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    noColourmenuItem_actionPerformed(e);
                }
            });
        wrapMenuItem.setLabel("Wrap");
        wrapMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    wrapMenuItem_actionPerformed(null);
                }
            });
        renderGapsMenuItem.setLabel("Show Gaps");
        renderGapsMenuItem.setState(true);
        renderGapsMenuItem.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    renderGapsMenuItem_actionPerformed(null);
                }
            });
        findMenuItem.setLabel("Find...");
        findMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    findMenuItem_actionPerformed(e);
                }
            });
        searchMenu.setLabel("Search");

        abovePIDThreshold.setLabel("Above Identity Threshold");
        abovePIDThreshold.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    abovePIDThreshold_actionPerformed(null);
                }
            });
        nucleotideColour.setLabel("Nucleotide");
        nucleotideColour.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    nucleotideColour_actionPerformed(e);
                }
            });
        deleteGroups.setLabel("Undefine Groups");
        deleteGroups.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    deleteGroups_actionPerformed(e);
                }
            });
        copy.setLabel("Copy");
        copy.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    copy_actionPerformed(e);
                }
            });
        cut.setLabel("Cut");
        cut.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    cut_actionPerformed(e);
                }
            });
        delete.setLabel("Delete");
        delete.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    delete_actionPerformed(e);
                }
            });
        Menu1.setLabel("Paste");
        pasteNew.setLabel("To New Alignment");
        pasteNew.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    pasteNew_actionPerformed(e);
                }
            });
        pasteThis.setLabel("Add To This Alignment");
        pasteThis.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    pasteThis_actionPerformed(e);
                }
            });
        applyToAllGroups.setLabel("Apply Colour To All Groups");
        applyToAllGroups.setState(true);
        applyToAllGroups.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    applyToAllGroups_actionPerformed(null);
                }
            });
        font.setLabel("Font...");
        font.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    font_actionPerformed(e);
                }
            });
        scaleAbove.setLabel("Scale Above");
        scaleAbove.setState(true);
        scaleAbove.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    scaleAbove_actionPerformed(null);
                }
            });
        scaleLeft.setEnabled(false);
        scaleLeft.setState(true);
        scaleLeft.setLabel("Scale Left");
        scaleLeft.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    scaleLeft_actionPerformed(null);
                }
            });
        scaleRight.setEnabled(false);
        scaleRight.setState(true);
        scaleRight.setLabel("Scale Right");
        scaleRight.addItemListener(new java.awt.event.ItemListener() {
                public void itemStateChanged(ItemEvent e) {
                    scaleRight_actionPerformed(null);
                }
            });
        modifyPID.setLabel("Modify Identity Threshold...");
        modifyPID.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    modifyPID_actionPerformed(e);
                }
            });
        modifyConservation.setLabel("Modify Conservation Threshold...");
        modifyConservation.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    modifyConservation_actionPerformed(e);
                }
            });
        sortByTreeMenu.setLabel("By Tree Order");
        sort.setLabel("Sort");
        calculate.setLabel("Calculate Tree");
        inputText.setLabel("Input from textbox");
        inputText.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    inputText_actionPerformed(e);
                }
            });

        helpMenu.setLabel("Help");
        documentation.setLabel("Documentation");
        documentation.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    documentation_actionPerformed(e);
                }
            });

        about.setLabel("About...");
        about.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    about_actionPerformed(e);
                }
            });
          seqLimits.setState(true);
    seqLimits.setLabel("Show Sequence Limits");
    seqLimits.addItemListener(new ItemListener()
    {
      public void itemStateChanged(ItemEvent e)
      {
        seqLimits_itemStateChanged(e);
      }
    });

    alignFrameMenuBar.add(fileMenu);
        alignFrameMenuBar.add(editMenu);
        alignFrameMenuBar.add(searchMenu);
        alignFrameMenuBar.add(viewMenu);
        alignFrameMenuBar.add(colourMenu);
        alignFrameMenuBar.add(calculateMenu);
        alignFrameMenuBar.add(helpMenu);
        fileMenu.add(inputText);
        fileMenu.add(outputTextboxMenu);
        fileMenu.addSeparator();
        fileMenu.add(closeMenuItem);
        editMenu.add(undoMenuItem);
        editMenu.add(redoMenuItem);
        editMenu.add(cut);
        editMenu.add(copy);
        editMenu.add(Menu1);
        editMenu.add(delete);
        editMenu.addSeparator();
        editMenu.add(selectAllSequenceMenuItem);
        editMenu.add(deselectAllSequenceMenuItem);
        editMenu.add(invertSequenceMenuItem);
        editMenu.add(deleteGroups);
        editMenu.addSeparator();
        editMenu.add(remove2LeftMenuItem);
        editMenu.add(remove2RightMenuItem);
        editMenu.add(removeGappedColumnMenuItem);
        editMenu.add(removeAllGapsMenuItem);
        editMenu.add(removeRedundancyMenuItem);
        searchMenu.add(findMenuItem);
        viewMenu.add(font);
        viewMenu.addSeparator();
        viewMenu.add(seqLimits);
        viewMenu.addSeparator();
    viewMenu.add(wrapMenuItem);
        viewMenu.add(scaleAbove);
        viewMenu.add(scaleLeft);
        viewMenu.add(scaleRight);
        viewMenu.addSeparator();
    viewMenu.add(viewBoxesMenuItem);
        viewMenu.add(viewTextMenuItem);
        viewMenu.add(colourTextMenuItem);
        viewMenu.add(renderGapsMenuItem);
        viewMenu.add(annotationPanelMenuItem);
        viewMenu.addSeparator();
        viewMenu.add(overviewMenuItem);
        colourMenu.add(applyToAllGroups);
        colourMenu.addSeparator();
        colourMenu.add(noColourmenuItem);
        colourMenu.add(clustalColour);
        colourMenu.add(BLOSUM62Colour);
        colourMenu.add(PIDColour);
        colourMenu.add(zappoColour);
        colourMenu.add(taylorColour);
        colourMenu.add(hydrophobicityColour);
        colourMenu.add(helixColour);
        colourMenu.add(strandColour);
        colourMenu.add(turnColour);
        colourMenu.add(buriedColour);
        colourMenu.add(nucleotideColour);
        colourMenu.add(userDefinedColour);
        colourMenu.addSeparator();
        colourMenu.add(conservationMenuItem);
        colourMenu.add(modifyConservation);
        colourMenu.add(abovePIDThreshold);
        colourMenu.add(modifyPID);
        calculateMenu.add(sort);
        calculateMenu.add(calculate);
        calculateMenu.addSeparator();
        calculateMenu.add(pairwiseAlignmentMenuItem);
        calculateMenu.add(PCAMenuItem);
        this.add(statusBar, BorderLayout.SOUTH);
        Menu1.add(pasteNew);
        Menu1.add(pasteThis);
        sort.add(sortIDMenuItem);
        sort.add(sortByTreeMenu);
        sort.add(sortGroupMenuItem);
        sort.add(sortPairwiseMenuItem);
        calculate.add(averageDistanceTreeMenuItem);
        calculate.add(neighbourTreeMenuItem);
        calculate.add(avDistanceTreeBlosumMenuItem);
        calculate.add(njTreeBlosumMenuItem);
        helpMenu.add(documentation);
        helpMenu.add(about);
    }
}

