/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package jalview.appletgui;

import java.awt.*;
import java.awt.event.*;

import jalview.datamodel.*;
import jalview.io.*;

public class CutAndPasteTransfer extends Panel
{
  jalview.bin.JalviewLite applet;

  boolean pdbImport = false;
  Sequence seq;
  SeqCanvas seqcanvas;

  public CutAndPasteTransfer(boolean forImport, jalview.bin.JalviewLite mainApplet)
  {
    try {
        jbInit();
    } catch (Exception e) {
        e.printStackTrace();
    }

    applet = mainApplet;

    if (!forImport)
    {
      buttonPanel.setVisible(false);
    }
  }

  public void setPDBImport(Sequence seq, SeqCanvas seqcanvas)
  {
    this.seq = seq;
    this.seqcanvas = seqcanvas;
    pdbImport = true;
  }


  public String getText()
  {
    return textarea.getText();
  }

  public void setText(String text)
  {
    textarea.setText(text);
  }

  protected void ok_actionPerformed(ActionEvent e)
  {
    String text = getText();
    int length = text.length();
    textarea.append("\n");
    if(textarea.getText().length()==length)
    {
      String warning = "\n\n#################################################\n"
          +"WARNING!! THIS IS THE MAXIMUM SIZE OF TEXTAREA!!\n"
          +"\nCAN'T INPUT FULL ALIGNMENT"
          +"\n\nYOU MUST DELETE THIS WARNING TO CONTINUE"
          +"\n\nMAKE SURE LAST SEQUENCE PASTED IS COMPLETE"
          +"\n#################################################\n";
      textarea.setText(text.substring(0, text.length()-warning.length())
          +warning);

      textarea.setCaretPosition(text.length());
    }

    if(pdbImport)
    {
      new MCview.AppletPDBViewer(text, "Paste", seq, seqcanvas);
    }
    else
    {
      SequenceI[] sequences = null;

      String format = IdentifyFile.Identify(text, "Paste");
      sequences = new AppletFormatAdapter().readFile(text, "Paste", format);

      if (sequences != null)
      {
        AlignFrame af = new AlignFrame(new Alignment(sequences), applet,
                                       "Cut & Paste input - " + format);
        af.statusBar.setText("Successfully pasted alignment file");
      }
    }

    ((Frame)this.getParent()).setVisible(false);
  }

  protected void cancel_actionPerformed(ActionEvent e)
  {
    ((Frame)this.getParent()).setVisible(false);
  }

  protected TextArea textarea = new TextArea();
  Button ok = new Button();
  Button cancel = new Button();
  protected Panel buttonPanel = new Panel();
  BorderLayout borderLayout1 = new BorderLayout();


  private void jbInit() throws Exception {
      textarea.setFont(new java.awt.Font("Monospaced", Font.PLAIN, 10));
      textarea.setText("Paste your alignment file here");
      textarea.addMouseListener(new java.awt.event.MouseAdapter() {
              public void mousePressed(MouseEvent e) {
                  textarea_mousePressed(e);
              }
          });
      this.setLayout(borderLayout1);
      ok.setLabel("OK");
      ok.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  ok_actionPerformed(e);
              }
          });
      cancel.setLabel("Cancel");
      cancel.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  cancel_actionPerformed(e);
              }
          });
      this.add(buttonPanel, BorderLayout.SOUTH);
      buttonPanel.add(ok, null);
      buttonPanel.add(cancel, null);
      this.add(textarea, java.awt.BorderLayout.CENTER);
  }

  void textarea_mousePressed(MouseEvent e) {
      if (textarea.getText().startsWith("Paste your")) {
          textarea.setText("");
      }
  }
}
