/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.appletgui;

import jalview.datamodel.*;

import java.awt.*;

import java.util.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.4 $
 */
public class FeatureRenderer
{
    AlignViewport av;
    SequenceGroup currentSequenceGroup = null;
    SequenceGroup[] allGroups = null;
    Color resBoxColour;
    Graphics graphics;
    float transparency = .4f;

    // The following vector holds the features which are
    // to be added, in the correct order or rendering
    Vector featuresDisplayed;

    /**
     * Creates a new FeatureRenderer object.
     *
     * @param av DOCUMENT ME!
     */
    public FeatureRenderer(AlignViewport av)
    {
        this.av = av;
        initColours();
        String version = System.getProperty("java.version");
        if (version.indexOf("1.1") == 0)
          highlightTransparent = false;

    }

boolean highlightTransparent = true;
    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     * @param seq DOCUMENT ME!
     * @param sg DOCUMENT ME!
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     * @param x1 DOCUMENT ME!
     * @param y1 DOCUMENT ME!
     * @param width DOCUMENT ME!
     * @param height DOCUMENT ME!
     */
    public void drawSequence(Graphics g, SequenceI seq, SequenceGroup[] sg,
        int start, int end, int x1, int y1, int width, int height)
    {

        if(seq.getSequenceFeatures()==null)
          return;

        Enumeration e = null, e2;
        String type;
        if(featuresDisplayed!=null)
          e = featuresDisplayed.elements();
        else
         e = seq.getSequenceFeatures().elements();

       if(highlightTransparent)
        {
        //  MyGraphics.SetTransparency(g, transparency);
        }


        while (e.hasMoreElements())
        {
            SequenceFeature sf=null;
            if(featuresDisplayed!=null)
            {
              e2 = seq.getSequenceFeatures().elements();
              type = e.nextElement().toString();
              while(e2.hasMoreElements())
              {
                sf = (SequenceFeature) e2.nextElement();
                if(sf.getType().equals(type))
                  break;
                else
                  sf = null;
              }
            }
            else
            {
              sf = (SequenceFeature) e.nextElement();
              type = sf.getType();
            }

            if(sf==null)
              continue;


            if (sf.getBegin() > seq.getEnd())
            {
                continue;
            }

            int fstart = seq.findIndex(sf.getBegin()) - 1;
            int fend = seq.findIndex(sf.getEnd()) - 1;

            if (((fstart <= end) && (fend >= start)))
            {
                if (fstart < start)
                { // fix for if the feature we have starts before the sequence start,
                    fstart = start; // but the feature end is still valid!!
                }

                if (fend >= end)
                {
                    fend = end;
                }

                if (fstart == fend)
                {
                    g.setColor(getColour(type));
                    g.fillRoundRect((fstart - start) * width, y1, width,
                        height, 4, 4);
                    g.setColor(Color.white);

                    char s = seq.getSequence().charAt(fstart);
                    FontMetrics fm = g.getFontMetrics();
                    int charOffset = (width - fm.charWidth(s)) / 2;
                    int pady = height / 5;
                    g.drawString(String.valueOf(s),
                        charOffset + x1 + (width * (fstart - start)),
                        (y1 + height) - pady);
                }
                else
                {
                    for (int i = fstart; i <= fend; i++)
                    {
                        char s = seq.getSequence().charAt(i);

                        if (jalview.util.Comparison.isGap(s))
                        {
                            continue;
                        }

                        g.setColor(getColour(type));
                        g.fillRect((i - start) * width, y1, width, height);

                        g.setColor(Color.white);

                        FontMetrics fm = g.getFontMetrics();
                        int charOffset = (width - fm.charWidth(s)) / 2;
                        int pady = height / 5;
                        g.drawString(String.valueOf(s),
                            charOffset + x1 + (width * (i - start)),
                            (y1 + height) - pady);
                    }
                }
            }
          }

          if (highlightTransparent)
          {
        //    MyGraphics.SetTransparency(g, 1.0f);
          }
    }

    public Color getColour(String featureType)
    {
      return (Color)featureColours.get(featureType);
    }

    public void setColour(String featureType, Color col)
    {
      featureColours.put(featureType, col);
    }

    public void setTransparency(float value)
    {
      transparency = value;
    }

    public float getTransparency()
    {
      return transparency;
    }

    public void setFeaturePriority(Object [][] data)
    {
      // The feature table will display high priority
      // features at the top, but theses are the ones
      // we need to render last, so invert the data
      featuresDisplayed = new Vector();
      for(int i=data.length-1; i>-1; i--)
      {
       String type = data[i][0].toString();
       setColour(type, (Color)data[i][1]);
       if( ((Boolean)data[i][2]).booleanValue() )
         featuresDisplayed.addElement(type);
      }
    }

    Hashtable featureColours = new Hashtable();
    void initColours()
    {
      featureColours.put("active site", new Color(255, 75, 0));
      featureColours.put("binding site", new Color(245, 85, 0));
      featureColours.put("calcium-binding region", new Color(235, 95, 0));
      featureColours.put("chain", new Color(225, 105, 0));
      featureColours.put("coiled-coil region", new Color(215, 115, 0));
      featureColours.put("compositionally biased region", new Color(205, 125, 0));
      featureColours.put("cross-link", new Color(195, 135, 0));
      featureColours.put("disulfide bond", new Color(185, 145, 0));
      featureColours.put("DNA-binding region", new Color(175, 155, 0));
      featureColours.put("domain", new Color(165, 165, 0));
      featureColours.put("glycosylation site", new Color(155, 175, 0));
      featureColours.put("helix", new Color(145, 185, 0));
      featureColours.put("initiator methionine", new Color(135, 195, 5));
      featureColours.put("lipid moiety-binding region", new Color(125, 205, 15));
      featureColours.put("metal ion-binding site", new Color(115, 215, 25));
      featureColours.put("modified residue", new Color(105, 225, 35));
      featureColours.put("mutagenesis site", new Color(95, 235, 45));
      featureColours.put("non-consecutive residues", new Color(85, 245, 55));
      featureColours.put("non-terminal residue", new Color(75, 255, 65));
      featureColours.put("nucleotide phosphate-binding region",
                         new Color(65, 245, 75));
      featureColours.put("peptide", new Color(55, 235, 85));
      featureColours.put("propeptide", new Color(45, 225, 95));
      featureColours.put("region of interest", new Color(35, 215, 105));
      featureColours.put("repeat", new Color(25, 205, 115));
      featureColours.put("selenocysteine", new Color(15, 195, 125));
      featureColours.put("sequence conflict", new Color(5, 185, 135));
      featureColours.put("sequence variant", new Color(0, 175, 145));
      featureColours.put("short sequence motif", new Color(0, 165, 155));
      featureColours.put("signal peptide", new Color(0, 155, 165));
      featureColours.put("site", new Color(0, 145, 175));
      featureColours.put("splice variant", new Color(0, 135, 185));
      featureColours.put("strand", new Color(0, 125, 195));
      featureColours.put("topological domain", new Color(0, 115, 205));
      featureColours.put("transit peptide", new Color(0, 105, 215));
      featureColours.put("transmembrane region", new Color(0, 95, 225));
      featureColours.put("turn", new Color(0, 85, 235));
      featureColours.put("unsure residue", new Color(0, 75, 245));
      featureColours.put("zinc finger region", new Color(0, 65, 255));
    }

}



