/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.datamodel.*;

import java.awt.*;

import java.util.*;

import java.awt.image.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.13 $
 */
public class FeatureRenderer
{
    AlignViewport av;
    Color resBoxColour;
    float transparency = 1.0f;
    FontMetrics fm;
    int charOffset;
    boolean drawText = true;

    // The following vector holds the features which are
    // to be added, in the correct order or rendering
    Vector featuresDisplayed = null;

    /**
     * Creates a new FeatureRenderer object.
     *
     * @param av DOCUMENT ME!
     */
    public FeatureRenderer(AlignViewport av)
    {
        this.av = av;
        initColours();
    }

    /**
     * This is used by the Molecule Viewer to get the accurate colour
     * of the rendered sequence
     */
    BufferedImage bi;
    public synchronized Color findFeatureColour(Color initialCol, SequenceI seq, int i)
    {
      if(!av.showSequenceFeatures)
        return initialCol;

      if (bi == null)
        bi = new BufferedImage(1, 1, BufferedImage.TYPE_INT_RGB);

      bi.setRGB(0,0, initialCol.getRGB());

      drawText = false;

      drawSequence(bi.getGraphics(), seq, i, i, 0, 0, 1, 1);
      drawText = true;

      return new Color(bi.getRGB(0, 0));
    }


    /**
     * DOCUMENT ME!
     *
     * @param g DOCUMENT ME!
     * @param seq DOCUMENT ME!
     * @param sg DOCUMENT ME!
     * @param start DOCUMENT ME!
     * @param end DOCUMENT ME!
     * @param x1 DOCUMENT ME!
     * @param y1 DOCUMENT ME!
     * @param width DOCUMENT ME!
     * @param height DOCUMENT ME!
     */
    public void drawSequence(Graphics g, SequenceI seq,
                             int start, int end, int x1, int y1, int width, int height)
    {

//System.out.println(start+" "+end+" "+x1+" "+y1);
      if (seq.getDatasetSequence().getSequenceFeatures() == null
          || seq.getDatasetSequence().getSequenceFeatures().size()==0)
        return;

      fm = g.getFontMetrics();

      if (transparency != 1)
      {
        Graphics2D g2 = (Graphics2D) g;
        g2.setComposite(
            AlphaComposite.getInstance(
                AlphaComposite.SRC_OVER, transparency));
      }

      String type;
      SequenceFeature sf;
      if (featuresDisplayed == null)
        findAllFeatures();

      Enumeration e = featuresDisplayed.elements(), e2;

      // Loop through each visible feature
      while (e.hasMoreElements())
      {

        type = e.nextElement().toString();
        e2 = seq.getDatasetSequence().getSequenceFeatures().elements();
        // loop through all features in sequence to find
        // current feature to render
          while (e2.hasMoreElements())
        {

          sf = (SequenceFeature) e2.nextElement();
          if (!type.equals(sf.getType()))
            continue;

          if (sf.getBegin() > seq.getEnd())
            continue;

          if (type.equals("disulfide bond"))
          {

            renderFeature(g, seq,
                          seq.findIndex(sf.getBegin()) - 1,
                          seq.findIndex(sf.getBegin()) - 1,
                          type, start, end, x1, y1, width, height);
            renderFeature(g, seq,
                          seq.findIndex(sf.getEnd()) - 1,
                          seq.findIndex(sf.getEnd()) - 1,
                          type, start, end, x1, y1, width, height);

          }
          else
            renderFeature(g, seq,
                          seq.findIndex(sf.getBegin()) - 1,
                          seq.findIndex(sf.getEnd()) - 1,
                          type, start, end, x1, y1, width, height);
        }
      }

        if(transparency!=1.0f)
        {
          Graphics2D g2 = (Graphics2D) g;
          g2.setComposite(
              AlphaComposite.getInstance(
                  AlphaComposite.SRC_OVER, 1.0f));
        }
    }


    void renderFeature(Graphics g, SequenceI seq,
                       int fstart, int fend, String type, int start, int end, int x1, int y1, int width, int height)
    {

      if (((fstart <= end) && (fend >= start)))
      {
          if (fstart < start)
          { // fix for if the feature we have starts before the sequence start,
              fstart = start; // but the feature end is still valid!!
          }

          if (fend >= end)
          {
            fend = end;
          }
          for (int i = fstart; i <= fend; i++)
          {
            char s = seq.getSequence().charAt(i);

            if (jalview.util.Comparison.isGap(s))
            {
              continue;
            }

            g.setColor(getColour(type));

            g.fillRect( (i - start) * width, y1, width, height);

            if(drawText)
           {
             g.setColor(Color.white);
             charOffset = (width - fm.charWidth(s)) / 2;
             g.drawString(String.valueOf(s),
                          charOffset + x1 + (width * (i - start)),
                          (y1 + height) - height / 5); //pady = height / 5;
           }
          }
        }
    }

    void findAllFeatures()
    {
      Vector features = new Vector();
      SequenceFeature sf;
      featuresDisplayed = new Vector();
      Enumeration e;
      for (int i = 0; i < av.alignment.getHeight(); i++)
      {
        features = av.alignment.getSequenceAt(i).getDatasetSequence().
            getSequenceFeatures();
        if (features == null)
          continue;

        e = features.elements();
        while (e.hasMoreElements())
        {
          sf = (SequenceFeature) e.nextElement();
          if (!featuresDisplayed.contains(sf.getType()))
          {
            featuresDisplayed.addElement(sf.getType());
          }
        }
      }
    }

    public Color getColour(String featureType)
    {
      return (Color)featureColours.get(featureType);
    }

    public void setColour(String featureType, Color col)
    {
      featureColours.put(featureType, col);
    }

    public void setTransparency(float value)
    {
      transparency = value;
    }

    public float getTransparency()
    {
      return transparency;
    }

    public void setFeaturePriority(Object [][] data)
    {
      // The feature table will display high priority
      // features at the top, but theses are the ones
      // we need to render last, so invert the data
      featuresDisplayed.clear();
      for(int i=data.length-1; i>-1; i--)
      {
       String type = data[i][0].toString();
       setColour(type, (Color)data[i][1]);
       if( ((Boolean)data[i][2]).booleanValue() )
         featuresDisplayed.addElement(type);
      }
    }

    Hashtable featureColours = new Hashtable();
    void initColours()
    {
      featureColours.put("active site", new Color(255, 75, 0));
      featureColours.put("binding site", new Color(245, 85, 0));
      featureColours.put("calcium-binding region", new Color(235, 95, 0));
      featureColours.put("chain", new Color(225, 105, 0));
      featureColours.put("coiled-coil region", new Color(215, 115, 0));
      featureColours.put("compositionally biased region", new Color(205, 125, 0));
      featureColours.put("cross-link", new Color(195, 135, 0));
      featureColours.put("disulfide bond", new Color(230,230,0));
      featureColours.put("DNA-binding region", new Color(175, 155, 0));
      featureColours.put("domain", new Color(165, 165, 0));
      featureColours.put("glycosylation site", new Color(155, 175, 0));
      featureColours.put("helix", new Color(145, 185, 0));
      featureColours.put("initiator methionine", new Color(135, 195, 5));
      featureColours.put("lipid moiety-binding region", new Color(125, 205, 15));
      featureColours.put("metal ion-binding site", new Color(115, 215, 25));
      featureColours.put("modified residue", new Color(105, 225, 35));
      featureColours.put("mutagenesis site", new Color(95, 235, 45));
      featureColours.put("non-consecutive residues", new Color(85, 245, 55));
      featureColours.put("non-terminal residue", new Color(75, 255, 65));
      featureColours.put("nucleotide phosphate-binding region",new Color(65, 245, 75));
      featureColours.put("peptide", new Color(55, 235, 85));
      featureColours.put("propeptide", new Color(45, 225, 95));
      featureColours.put("region of interest", new Color(35, 215, 105));
      featureColours.put("repeat", new Color(25, 205, 115));
      featureColours.put("selenocysteine", new Color(15, 195, 125));
      featureColours.put("sequence conflict", new Color(5, 185, 135));
      featureColours.put("sequence variant", new Color(0, 175, 145));
      featureColours.put("short sequence motif", new Color(0, 165, 155));
      featureColours.put("signal peptide", new Color(0, 155, 165));
      featureColours.put("site", new Color(0, 145, 175));
      featureColours.put("splice variant", new Color(0, 135, 185));
      featureColours.put("strand", new Color(0, 125, 195));
      featureColours.put("topological domain", new Color(0, 115, 205));
      featureColours.put("transit peptide", new Color(0, 105, 215));
      featureColours.put("transmembrane region", new Color(0, 95, 225));
      featureColours.put("turn", new Color(0, 85, 235));
      featureColours.put("unsure residue", new Color(0, 75, 245));
      featureColours.put("zinc finger region", new Color(0, 65, 255));
    }

}
